#include "exports.h"
#include "file.h"
#include "linkablemapobj.h"
#include "misc.h"
#include "texteditor.h"
#include "mainwindow.h"

extern Main *mainWindow;

ExportBase::ExportBase()
{
	indentPerDepth="  ";
}

void ExportBase::setPath (const QString &p)
{
	filepath=p;
}

void ExportBase::setMapCenter(MapCenterObj *mc)
{
	mapCenter=mc;
}

QString ExportBase::getSectionString(BranchObj *bostart)
{
	QString r;
	BranchObj *bo=bostart;
	int depth=bo->getDepth();
	while (depth>0)
	{
		r=QString("%1").arg(1+bo->getNum(),0,10)+"." + r;
		bo=(BranchObj*)(bo->getParObj());
		depth=bo->getDepth();
	}	
	if (r.isEmpty())
		return r;
	else	
		return r + " ";
}

void ExportBase::exportXML()
{
	QFile file (filepath);
	if ( !file.open( IO_WriteOnly ) )
	{
		// FIXME experimental, testing
		cout << "ExportBase::exportXML  couldn't open "<<filepath<<endl;
		return;
	}
	QTextStream ts( &file );	// use LANG decoding here...

	// Main loop over all branches
	QString s;
	QString actIndent("");
	int i;
	uint j;
	BranchObj *bo;
	bo=mapCenter->first();
	while (bo) 
	{
		// Make indentstring
		for (i=0;i<bo->getDepth();i++) actIndent+= indentPerDepth;

		// Write heading
		//	write (actIndent + getSectionString(bo) + bo->getHeading()+ "\n");
		if (bo->getDepth()==0)
		{
			ts << (bo->getHeading()+ "\n");
			for (j=0;j<bo->getHeading().length();j++) ts<<"=";
			ts << "\n";
		} else 	if (bo->getDepth()==1)
			ts << ("\n"+getSectionString(bo) + bo->getHeading()+ "\n");
		else	if (bo->getDepth()==2)
			ts << (actIndent + " o " + bo->getHeading()+ "\n");
		else	
			ts << (actIndent + " - " + bo->getHeading()+ "\n");
		
		// If necessary, write note
		if (!bo->getNote().isEmpty())
		{
			s =textConvertToASCII(bo->getNote());
			s=s.replace ("\n","\n"+actIndent);
			ts << (s+"\n\n");
		}
		
		bo=bo->next();
		actIndent="";
	}
	file.close();
}

// Exports a map to a LaTex file.  This file needs to be included or inported into a LaTex document
// it will not add a preamble, or anything that makes a full LaTex document.
void ExportLaTeX::exportLaTeX() 
{
  QFile file (filepath);
  if ( !file.open( IO_WriteOnly ) ) {
    // FIXME
    cout << "Export::exportMap  couldn't open "<<filepath<<endl;
    return;
  }
  QTextStream ts( &file );	// use LANG decoding here...
  ts.setEncoding (QTextStream::UnicodeUTF8); // Force UTF8
  
  // Main loop over all branches
  QString s;
  // QString actIndent("");
  // int i;
  BranchObj *bo;
  bo=mapCenter->first();
  while (bo) {
    if (bo->getDepth()==0);
    else if (bo->getDepth()==1) {
      ts << ("\\chapter{" + bo->getHeading()+ "}\n");
    }
    else if (bo->getDepth()==2) {
      ts << ("\\section{" + bo->getHeading()+ "}\n");
    }
    else if (bo->getDepth()==3) {
      ts << ("\\subsection{" + bo->getHeading()+ "}\n");
    }
    else if (bo->getDepth()==4) {
      ts << ("\\subsubsection{" + bo->getHeading()+ "}\n");
    }
    else {
      ts << ("\\paragraph*{" + bo->getHeading()+ "}\n");
    }
    
    // If necessary, write note
    if (!bo->getNote().isEmpty()) {
      ts << (textConvertToASCII(bo->getNote()));
      ts << ("\n");
    }
    
    bo=bo->next();
   }
  file.close();
}

ExportOO::ExportOO()
{
	// Create tmpdir and set paths
	tmpDir.setPath (makeUniqueDir("/tmp/vym-XXXXXX"));
	templateDir="oo-test/suse-template/";
	contentTemplateFile=templateDir+"content-template.xml";
	contentFile=tmpDir.path()+"/content.xml";
	pageTemplateFile=templateDir+"page-template.xml";
	chapterTemplateFile=templateDir+"chapter-template.xml";

	outputFile=tmpDir.currentDirPath()+"/out.odp";
	
	// Copy template to tmpdir
	system ("cp -r "+templateDir+"* "+tmpDir.path());
	

	// Read content-template
	if (!loadStringFromDisk (contentTemplateFile,content))
	{
		qWarning ("Export::exportOOPresentation() Couldn't load from "+contentTemplateFile);
		return;
	}

	// Read page-template
	QString pageTemplate;
	if (!loadStringFromDisk (pageTemplateFile,pageTemplate))
	{
		qWarning ("Export::exportOOPresentation() Couldn't load from "+pageTemplateFile);
		return;
	}
	
	// Read chapter-template
	QString chapterTemplate;
	if (!loadStringFromDisk (chapterTemplateFile,chapterTemplate))
	{
		qWarning ("Export::exportOOPresentation() Couldn't load from "+chapterTemplateFile);
		return;
	}
}

ExportOO::~ExportOO()
{
	// Remove tmpdir
	//FIXME removeDir (tmpDir);
	cout << "tmpDir="<<tmpDir.path()<<endl;
}	

QString ExportOO::buildList (BranchObj *current)
{
    QString r;
    BranchObj *bo;
    int i=0;
    bo=current->getFirstBranch();
    if (bo)
    {
        // Start list
        r+="<text:list>\n";
        while (bo)
        {
            r+="<text:list-item><text:p text:style-name=\"P7\">\n  "+
               bo->getHeading()+
               "\n</text:p></text:list-item>\n";
        }
        r+="</text:list>\n";
    }
    return r;
}

QString ExportOO::walkPages (BranchObj *current)
{
	//FIXME Denkfehler: mit current anfangen, nicht mit
	// Kind von current. Sonst erwisch ich nur jede 2. Depth
	//


    QString r;
    BranchObj *bo;
    int i=0;
    bo=current->getFirstBranch();
    while(bo)
    {
		r+=quotemeta(bo->getHeading())+"\n";
		if (bo->getFirstBranch()) 
			r+=walkPages (bo->getFirstBranch());
		i++;
		bo=current->getBranchNum (i);
    }
    return r;
}


void ExportOO::exportPresentation()
{
	QString allPages=walkPages (mapCenter);

	cout << allPages<<endl;
	
	// Insert new content
	content.replace ("<!-- INSERT TITLE -->",quotemeta(mapCenter->getHeading()));
	content.replace ("<!-- INSERT AUTHOR -->",quotemeta(mapCenter->getAuthor()));
	content.replace ("<!-- INSERT PAGES -->",allPages);

	// Write modified content
	QFile f (contentFile);
    if ( !f.open( IO_WriteOnly ) ) 
	{
		mainWindow->statusMessage(QString(QObject::tr("Could not write to %1")).arg(outputFile));
		return;
    }

    QTextStream t( &f );
    t << content;
    f.close();

	mainWindow->statusMessage( QString( QObject::tr("Map exported as %1")).arg( outputFile));


	// zip tmpdir to destination
	zipDir (tmpDir,outputFile);	

	/* FIXME not needed
	QString onePage;
	QString allPages;
	QString heading;


	// Walk through chapters (mainbranches)
	QString s;

	QString list;
	BranchObj *bo;
	BranchObj *boLastPage=NULL;
	bo=mapCenter->first();
	int depth;
	int lastDepth=-1;
	int basePageDepth=2;
	while (bo)
	{
		depth=bo->getDepth();
		heading=quotemeta(bo->getHeading());
		if (depth>1)
		{
			if (depth==basePageDepth)
			{
				// Start a new page containing a heading and a list
				if (boLastPage==NULL)
				{
					// Start new single page
					boLastPage=bo;
					onePage=pageTemplate;
					onePage.replace ("<!-- INSERT PAGE HEADING -->", heading);
					list="";
				} else
				{
					// Finish last page, start new one
					onePage.replace ("<!-- INSERT LIST -->", list);
					boLastPage=bo;
					allPages+=onePage;
					onePage=pageTemplate;
					onePage.replace ("<!-- INSERT PAGE HEADING -->", heading);
					list="";
				}
			} else
			{
				if (lastDepth==depth-1)
					// entering deeper branches, new list
					list+="<text:list>";
				// Just add heading to list on current page
				list+="<text:list-item><text:p text:style-name=\"P7\">"+heading+"</text:p></text:list-item>\n";
				if (lastDepth==depth+1)
					// returning from deeper branches, finish list
					list+="</text:list>";
			}
		}	
		lastDepth=depth;
		bo=bo->next();
	}
*/
}

